#include <errno.h>
#include <fcntl.h>
#include <ndbm.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "lutil.h"
#include "rfcmsg.h"
#include "xutil.h"

#define ifhistdb "/fnet/spool/ifhist"

static DBM *db_id = NULL;
static int opened = 0;

static int db_init(void)
{
    if (opened == -1) return 0;
    if (opened ==  1) return 1;
    if ((db_id = dbm_open (ifhistdb, O_RDWR|O_CREAT, 0660)) == NULL)
    {
	logerr ("$Could non open database file \"%s\".", ifhistdb);
	opened = -1;
	return 0;
    }
    opened = 1;
    return 1;
}

void hist_save (msgid, msg)
char *msgid;
rfcmsg *msg;
{
    datum key, val;
    char *p, *q;
    if (msg   == NULL) return;
    if (msgid == NULL) return;
    if ((p = hdr ("MSGID", msg)) == NULL) return;
    if (!db_init()) return;
    key.dptr = xstrcpy (p);
    if (*(p = key.dptr + strlen (key.dptr) - 1) == '\n') *p = '\0';
    key.dsize = strlen(key.dptr);
    q = xstrcpy (msgid);
    if (*(p = q + strlen (q) - 1) == '\n') *p = '\0';
    p = "%08lx\t%s";
    val.dptr = xmalloc (strlen (p) + 8 + strlen (q));
    sprintf (val.dptr, p, time (NULL), q);
    val.dsize = strlen (val.dptr);
    if (dbm_store (db_id, key, val, DBM_REPLACE))
	logerr ("$cannot store: \"%s\" \"%s\"", msgid, "");
    free (key.dptr);
    free (val.dptr);
    free (q);
}

char *hist_fetch(msgid)
char *msgid;
{
    static char *str = NULL;
    datum key, val;
    char *p;
    if (!msgid) 	return NULL;
    if (!db_init())	return NULL;
    key.dptr = xstrcpy (msgid);
    if (*(p = key.dptr + strlen (key.dptr) - 1) == '\n') *p = '\0';
    key.dsize = strlen (key.dptr);
    val = dbm_fetch(db_id, key);
    free (key.dptr);
    if (!val.dptr) return NULL;
    if (str) {free (str); str = NULL;}
    str = xmalloc (val.dsize + 1);
    strncpy (str, val.dptr, val.dsize);
    str [val.dsize] = '\0';
    p = strchr (str, '\t');
    return p ? p + 1 : str;
}
